#!/bin/bash

function usage {
      [[ ${1:-} = "-s" ]] || echo "Запустить команду docker compose в одном, нескольких или во всех сервисах"
      echo "Использование:"
      echo "    do.sh [-r] service[,service,...]|--all command"
      echo "          service - каталог установленного сервиса."
      echo "          --all   - выполнить в сервисах, указанных в order"
      echo "          -r      - c использованием --all, в обратном порядке"
      echo "          command - команда docker compose c опциями при необходимости"
      echo "    do.sh -h    Отобразить справку"
      exit 1
}

# DEBUG options
if [[ -n ${DEBUG:-} ]]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

workdir=$(dirname $(realpath -s $0))
cd "$workdir" || exit 1

PROGRAM=${0##*/}

while getopts ':hr-:' KEY; do
  case $KEY in
    h ) usage ;;
    r ) reverse='1!G;h;$!d;' ;;
    - )
        case $OPTARG in
          help )    usage ;;
          reverse ) reverse='1!G;h;$!d;' ;;
          all )     all_services=true ;;
          * )
              echo "Не верная опция $OPTARG при вызове $PROGRAM"
              usage -s; exit 1
          ;;
        esac
    ;;
    * )
        echo "Не верная опция $OPTARG при вызове $PROGRAM"
        usage -s; exit 1
    ;;
  esac
done
shift $((OPTIND -1))

[[ ! -f $workdir/.order ]] && cp "$workdir"/template.order "$workdir"/.order

filter_pattern='/^$/d;/^\s*#.*/d;s/\s*(\w+).*/\1/g;'$reverse
if [[ ${all_services:-} ]]; then
  mapfile -t services < <(sed -r "$filter_pattern" .order)
else
  if [[ $1 =~ .+,.+.* ]]; then
    OIFS=$IFS
    IFS=','
    services=( $1 )
    IFS=$OIFS
  else
    services[0]=$1
  fi
  shift
fi

cmnd=($@)

if [ $# -ne 0 ]; then
  if [[ ${cmnd[*]} =~ down.*|rm.*|stop.* ]] && [[ ${all_services:-} = true ]]; then
    echo -n "Удаляем контейнеры плагинов PromUC..."
    for container in $(docker ps -a -q -f name=plugin_); do
      docker rm -f -v "$container" &>/dev/null || { echo -e "FAIL\nОшибка, повторите операцию"; exit 2; }
    done
    echo "OK"
  fi
  for service in "${services[@]}"; do
    [[ -d $workdir/$service ]] || continue
    cd "$workdir"/"$service" || continue
    echo -n "Выполнение docker compose " "${cmnd[@]}" " для $service..."
    docker compose "${cmnd[@]}" &>/dev/null || { echo "FAIL"; continue; }
    echo "OK"
  done
else
  echo "Не указана docker-команда для выполнения"
  usage -s; exit 1
fi
