#!/bin/bash

# DEBUG options
if [ ! -z ${DEBUG:-} ]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

set -euo pipefail

echo -e "\nНастраиваем framework"

workdir=$(dirname $(realpath -s $0))
installdir=$(dirname $workdir)
source "$installdir"/install/functions.sh &> /dev/null || { echo -e "FAIL\nОшибка подключения библиотеки функций из файла $installdir/install/functions.sh"; exit 1; }
cd $workdir

if [ ! -f .env ] || [[ ${1:-} = "--recreate" ]] || [[ ${1:-} = "-r" ]]; then
  cp template.env .env
  source .env
  [ -f ../.env ] && source ../.env
  
  # from install/functions.sh
  set_service_domain
  host_add fw
  
  [ ! -f docker-compose.yml ] && cp docker-compose-cert.yml docker-compose.yml
  [ $(docker compose ps -aq | wc -l) -gt 0 ] && docker compose down -v
  
  echo -e "\nСоздаём базу данных framework..."
  [ $(docker ps -q -f name=postgres | wc -l) -eq 0 ] && { echo "Не запущен сервис Postgres"; exit 1; }
  echo -n "Проверяем готовность postgres..."
  for ((timer=0; timer < 61; timer++)); do
    docker exec -u postgres postgres pg_ctl status | grep "pg_ctl: server is running (PID: 1)" &> /dev/null && { echo "OK"; break; }
    [ $timer -eq 60 ] || { sleep 1; continue; }
    echo -e "\n"
    read -r -p "Готовность postgres не подтверждена в течении минуты. Повторить запрос? [Y/n]: " ANSWER
    [[ "${ANSWER:-y}" =~ ^[yY]$ ]] &&  timer=0 || exit 1
  done
  echo -n "Инициализируем базу данных framework..."
  docker cp conf/frameworkdb-init.sql postgres:/docker-entrypoint-initdb.d/
  docker exec postgres psql -U postgres -f /docker-entrypoint-initdb.d/frameworkdb-init.sql 1> /dev/null && echo 'OK' || { echo -e "FAIL\nОшибка создания базы данных"; exit 1; }
  
  source $installdir/proxy/.env
  case ${CERT:-1} in
    2 ) cp docker-compose-le.yml docker-compose.yml ;;
    * ) cp docker-compose-cert.yml docker-compose.yml ;;
  esac
else
  echo -e "Возможно имеются данные, установка игнорируется.\nДля принудительной установки удалите файл $workdir/.env или используйте флаг --recreate\n"
fi

[ $(docker image ls -q psm/promuc_framework:latest | wc -l) -eq 0 ] && { $installdir/getimage.sh psm/promuc_framework:latest || exit 1; }

echo -e "\nЗапускаем promuc_framework"
docker compose up -d --force-recreate --wait || { echo "Ошибка при запуске контейнера. Повторите установку."; exit 2; }


