#!/bin/bash

# DEBUG options
if [ ! -z ${DEBUG:-} ]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

set -euo pipefail

echo -e "\nНастраиваем gis"

workdir=$(dirname $(realpath -s $0))
installdir=$(dirname $workdir)
source "$installdir"/install/functions.sh &> /dev/null || { echo -e "FAIL\nОшибка подключения библиотеки функций из файла $installdir/install/functions.sh"; exit 1; }
cd $workdir || exit 1

if [ ! -f .env ] || [ ! -f data/db/planet-import-complete ] || [[ ${1:-} = "--recreate" ]] || [[ ${1:-} = "-r" ]]; then
  cp template.env .env
  source .env
  sed -r -i "s/^(REGION_PBF=).*/\1${REGION_PBF:-sochi}/" .env
  sed -r -i "s/^(OSM_TILE_SERVER_TAG=)[[:alnum:]\-\_]*/\1${OSM_TILE_SERVER_TAG:-latest}/" .env
  [ -f ../.env ] && source ../.env
  
  # from install/functions.sh
  set_service_domain
  host_add gis

  source .env

  echo "Удаляем данные и повторно импортируем..."
  [ ! -f docker-compose.yml ] && cp docker-compose-cert.yml docker-compose.yml
  [ $(docker compose ps -aq | wc -l) -gt 0 ] && docker compose down
  rm -rf data/{db,tiles}
  mkdir -p data/{db,tiles,extdata}  
  "$installdir"/getimage.sh "$REGION_PBF".pbf
  "$installdir"/getimage.sh simplified-water-polygons-split-3857.zip
  "$installdir"/getimage.sh water-polygons-split-3857.zip
  "$installdir"/getimage.sh antarctica-icesheet-outlines-3857.zip
  "$installdir"/getimage.sh antarctica-icesheet-polygons-3857.zip
  "$installdir"/getimage.sh ne_110m_admin_0_boundary_lines_land.zip
  chown -R 1000:1000 data conf

  echo "Импортируем данные из файла $installdir/images/$REGION_PBF.pbf"
  cpucount=$(grep -c ^processor /proc/cpuinfo 2>/dev/null)
  [[ $cpucount -gt 6 ]] && ((cpucount-=2)) || cpucount=4
  cachecount=$(($(awk '/MemAvailable/ { printf "%.0f \n", $2/1024 }' /proc/meminfo)/800))
  [[ $cachecount -gt 2 ]] && ((cachecount-=1)) || cachecount=1
  [ $(docker image ls -q psm/openstreetmap-tile-server:$OSM_TILE_SERVER_TAG | wc -l) -eq 0 ] && { $installdir/getimage.sh psm/openstreetmap-tile-server:$OSM_TILE_SERVER_TAG || exit 1; }
  docker run --rm ${http_proxy:+-e http_proxy} ${https_proxy:+-e https_proxy} -e THREADS=$cpucount -e "OSM2PGSQL_EXTRA_ARGS=-C $((cachecount*800))" -e AUTOVACUUM=off -v $installdir/images/$REGION_PBF.pbf:/data/region.osm.pbf -v $workdir/data/db:/data/database/ -v $workdir/data/tiles:/data/tiles -v $workdir/data/extdata:/data/extdata -v $workdir/conf/external-data.yml:/home/renderer/src/openstreetmap-carto/external-data.yml psm/openstreetmap-tile-server:$OSM_TILE_SERVER_TAG import

  source $installdir/proxy/.env
  case ${CERT:-1} in
    2 ) cp docker-compose-le.yml docker-compose.yml ;;
    * ) cp docker-compose-cert.yml docker-compose.yml ;;
  esac
else
  echo -e "Возможно имеются данные, установка игнорируется.\nДля принудительной установки используйте флаг --recreate\n"
fi

if [[ $(grep -c '#- gis.${SERVICE_DOMAIN:-local}' $installdir/proxy/docker-compose.yml) -gt 0 ]]; then
  sed -i "s/#- gis/- gis/g" $installdir/proxy/docker-compose.yml
  cd $installdir/proxy || exit 1
  if [ $(docker compose ps -aq | wc -l) -gt 0 ]; then
    echo "Перезапускаем сервис proxy..."
    success=1
    while [ ${success:-1} -eq 1 ]; do
      docker compose up -d --force-recreate 1> /dev/null && success=0 || { echo "Ошибка рестарта"; success=1; }
      if [ $success -ne 0 ]; then
        read -r -p "Повторить попытку? [Y/n]: " ANSWER
        [[ "${ANSWER:-y}" =~ ^[yY]$ ]] || exit 1
      fi
    done
  fi
fi
cd $workdir

[ $(docker image ls -q psm/openstreetmap-tile-server:$OSM_TILE_SERVER_TAG | wc -l) -eq 0 ] && { $installdir/getimage.sh psm/openstreetmap-tile-server:$OSM_TILE_SERVER_TAG || exit 1; }

echo -e "\nЗапускаем gis\n"
docker compose up -d --force-recreate --wait || { echo "Ошибка при запуске контейнера. Повторите установку."; exit 2; }
