#!/bin/bash 

# Скрипт не предназначен для самостоятельного запуска
# Скрипт предназначен для получения, проверки и токена администратора Gitea

[[ "${0##*/}" == "check.sh" ]] && exit 1

get_token() (
  source $installdir/gitea/.env
  if ! responseCode=$(\
    curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X POST \
      https://$GITEA_ADMIN_USER:$GITEA_ADMIN_PASS@git.$SERVICE_DOMAIN/api/v1/users/$GITEA_ADMIN_USER/tokens \
    -H 'accept: application/json'       \
    -H "Content-Type: application/json" \
    -d '{
          "name":"api"
        }' \
  ); then
    echo -e "FAIL\nОшибка при получении токена\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"
    exit 1
  fi
  if [ $responseCode -eq 201 ]; then
    echo "ОК"
    token=$(cat .apiresponse | jq -j .sha1)
  else
    echo -e "FAIL\nОшибка при получении токена\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"
    exit 1
  fi
  echo "GITEA_USER_TOKEN=$token" >> $installdir/gitea/.env
  echo $token > "$installdir"/gitea/.secrets/gitea_user_token
  echo $token > .secrets/chpt
)

check_token() {
  while :; do
    if ! responseCode=$(\
      curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X GET \
        "https://git.$SERVICE_DOMAIN/api/v1/version?token=$token" \
        -H 'accept: application/json'); then
      echo -e "Ошибка при обращении к API Gitea c токеном\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"
      read -r -p "Повторить? (Y/n): " ANSWER
      [[ "${ANSWER:-y}" =~ ^[yY]$ ]] && continue || { echo "Проверьте доступность Gitea по адресу https://git.$SERVICE_DOMAIN и повторите установку Updater"; exit 1; }
    fi
    break
  done
  if [ "$responseCode" -ne 200 ]; then
    echo -n "Токен Gitea не действителен, получаем новый..."
    get_token
  fi
}

check_token_exist() {
  if [ -f "$installdir"/gitea/.secrets/gitea_user_token ] && [[ $(wc -l < "$installdir"/gitea/.secrets/gitea_user_token) -eq 1 ]]; then
    token=$(< "$installdir"/gitea/.secrets/gitea_user_token)
    check_token
  else
    [ -f "$installdir"/gitea/.env ] || { echo -e "FAIL\nНе доступен файл .env в каталоге $installdir/gitea"; exit 2; }
    source "$installdir"/gitea/.env 
    if [ -z "${GITEA_USER_TOKEN:-}" ]; then
      if [ -z "${GITEA_ADMIN_USER:-}" ] || [ -z "${GITEA_ADMIN_PASSWORD:-}" ]; then
        echo -e "FAIL\nНе доступны данные пользователя Gitea, повторите установку Gitea"
        exit 2
      fi
      get_token
    else
      token=$GITEA_USER_TOKEN
      check_token
    fi
  fi
}
