#!/bin/bash

# DEBUG options
if [ -n "${DEBUG:-}" ]; then
  exec 5> "$(dirname "$(realpath -s "$0")")"/."$(basename "$0")".log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

set -euo pipefail

echo -e "\nНастраиваем gitea"
workdir=$(dirname "$(realpath -s "$0")")
installdir=$(dirname "$workdir")
source "$installdir"/install/functions.sh &> /dev/null || { echo -e "FAIL\nОшибка подключения библиотеки функций из файла $installdir/install/functions.sh"; exit 1; }
cd "$workdir" || exit 1

if [ ! -d data ] || [[ ${1:-} = "--recreate" ]] || [[ ${1:-} = "-r" ]]; then 
  [ ! -f docker-compose.yml ] && cp docker-compose-cert.yml docker-compose.yml
  [ "$(docker compose ps -aq | wc -l)" -gt 0 ] && docker compose down -v
  [ -f "$installdir"/.env ] && source "$installdir"/.env
  cp template.env .env
  
  # from install/functions.sh
  set_service_domain
  host_add git

  if [ "$(docker image ls -q gitea/gitea:"${GITEA_TAG:-1.20.2}" | wc -l)" -eq 0 ]; then
    "$installdir"/getimage.sh gitea/gitea:"${GITEA_TAG:-1.20.2}" || exit 1
  fi

  # создаём таблицу
  echo -n "Проверяем доступность postgres..."
  [ "$(docker ps -q -f name=postgres | wc -l)" -eq 0 ] && { echo -e"FAIL\nНе запущен сервис Postgres"; exit 1; }
  for ((timer=0; timer < 61; timer++)); do
    docker exec -u postgres postgres pg_ctl status | grep "pg_ctl: server is running (PID: 1)" &> /dev/null && { echo "OK"; break; }
    [ $timer -eq 60 ] || { sleep 1; continue; }
    echo -e "\n"
    read -r -p "Готовность postgres не подтверждена в течении минуты. Повторить запрос? [Y/n]: " ANSWER
    [[ "${ANSWER:-y}" =~ ^[yY]$ ]] &&  timer=0 || exit 1
  done
  echo -n "Инициализируем базу данных gitea..."
  docker cp conf/giteadb-init.sql postgres:/docker-entrypoint-initdb.d/ &>/dev/null
  docker exec postgres psql -U postgres -f /docker-entrypoint-initdb.d/giteadb-init.sql 1> /dev/null || { echo -e "FAIL\nОшибка создания базы данных"; exit 1; }
  echo 'OK'
  
  setup=0
  rm -rf data/
  mkdir -p data/gitea/conf/
  cp template.app.ini data/gitea/conf/app.ini
  sed -i -r "s/(SECRET_KEY = ).*/\1$(docker run -it --rm gitea/gitea:"$(sed -n 's/GITEA_TAG=//p' .env)" gitea generate secret SECRET_KEY | tr -dc '[:print:]')/" ./data/gitea/conf/app.ini
  chown -R 998:998 data/
  
  source $installdir/proxy/.env
  case ${CERT:-1} in
    2 ) cp docker-compose-le.yml docker-compose.yml ;;
    * ) cp docker-compose-cert.yml docker-compose.yml ;;
  esac

  sed -i -r "s/(GITEA_ADMIN_USER=).*/\1$(sed -n "s|TRAEFIK_ADMIN_LOGIN=||p" "$installdir"/proxy/.env | tr -d '\n')/" .env
  sed -i -r "s/(GITEA_ADMIN_PASS=).*/\1$(sed -n "s|TRAEFIK_ADMIN_PASS=||p" "$installdir"/proxy/.env | tr -d '\n')/" .env

else
  echo -e "Возможно имеются данные, установка игнорируется.\nДля принудительной установки используйте флаг --recreate\n"
  setup=1
fi

source .env

echo -e "\nЗапускаем gitea"
[ "$(docker image ls -q gitea/gitea:"${GITEA_TAG:-1.20.2}" | wc -l)" -eq 0 ] && { "$installdir"/getimage.sh gitea/gitea:"${GITEA_TAG:-1.20.2}" || exit 1; }
if docker compose up -d --force-recreate --wait; then
  if [ $setup -eq 0 ]; then
    source setup.sh
  fi
else
  echo "Ошибка при запуске контейнера. Повторите установку."
  exit 1
fi 
