#!/bin/bash

failed () {
  echo "Настройка прервана"
  rm -f .apiresponse
  exit 1
}

if [ -z $(which jq) ] || [ -z $(which curl) ]; then
  echo "Устанавливаем jq и curl"
  { apt-get update 2>&1 || echo E: update failed; } | grep -q '^[WE]:' && echo "Ошибка обновления, возможны ошибки при установке необходимых пакетов"
  apt-get -qq install jq curl 1> /dev/null || { echo "Ошибка установки"; exit 1; }
fi 

echo -en "\nОжидаем доступность API Gitea..."
timer=0
while [ $timer -lt 60 ]; do
  (( timer+=1 ))
  responseCode=$(\
    curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X GET \
    https://git.$SERVICE_DOMAIN/api/v1/version \
    -H 'accept: application/json'
  ) && { [[ "$responseCode" =~ (200|403) ]] && { echo "OK"; break; }; }
  [ $timer -eq 60 ] || { sleep 1; continue; }
  echo "FAIL"
  read -r -p "API Gitea не доступно в течении минуты, повторить запрос? (Y/n): " ANSWER
  [[ "${ANSWER:-y}" =~ ^[yY]$ ]] && timer=0 || exit 1
done

echo -n "Добавляем администратора..."
docker exec -u git gitea gitea admin user create --admin --username $GITEA_ADMIN_USER --password $GITEA_ADMIN_PASS --email $GITEA_ADMIN_USER@$SERVICE_DOMAIN &>/dev/null || { echo -e "FAIL\nОшибка при создании пользователя"; failed; }
echo "OK"
echo -e "\n\tлогин: $GITEA_ADMIN_USER\n\tпароль $GITEA_ADMIN_PASS\n"

token=$(sed -nr 's/GITEA_USER_TOKEN=(.+)/\1/p' .env)
if [ -z $token ]; then
  echo -n "Отсутствует токен для подключения к Gitea, получаем..."
  responseCode=""
  if ! responseCode=$(\
    curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X POST \
    https://$GITEA_ADMIN_USER:$GITEA_ADMIN_PASS@git.$SERVICE_DOMAIN/api/v1/users/$GITEA_ADMIN_USER/tokens \
    -H 'accept: application/json'       \
    -H "Content-Type: application/json" \
    -d '{
        "name":"setup",
        "scopes": [
          "write:admin",
          "read:misc",
          "write:organization",
          "write:repository",
          "write:user"
        ]
        }' \
  ); then
    echo -e "FAIL\nОшибка при получении токена\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"
    exit 1
  fi
  if [[ "$responseCode" == "201" ]]; then
    echo "OK"
    token=$(cat .apiresponse | jq -j .sha1)
  else
    echo -e "FAIL\nОшибка при получении токена\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"
    exit 1
  fi
  echo "GITEA_USER_TOKEN=$token" >> $installdir/gitea/.env
  mkdir -p "$installdir"/gitea/.secrets
  echo $token > "$installdir"/gitea/.secrets/gitea_user_token
fi

echo -n "Ограничение видимости $GITEA_ADMIN_USER..."
if ! responseCode=$(\
  curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X PATCH \
  https://git.$SERVICE_DOMAIN/api/v1/admin/users/$GITEA_ADMIN_USER?token=$token \
  -H 'accept: application/json' \
  -H 'Content-Type: application/json' \
  -d '{
    "login_name": "'$GITEA_ADMIN_USER'",
    "visibility": "private"
  }'
); then
  echo "FAIL"
  exit 1
fi
[[ "$responseCode" == "200" ]] || { echo "FAIL"; exit 1; }
echo "OK"

echo -e "\n### Gitea ###" >> ../.config
echo "логин:  $GITEA_ADMIN_USER" >> $installdir/.config
echo "пароль: $GITEA_ADMIN_PASS" >> $installdir/.config

source .env
rm -rf .apiresponse