#!/bin/bash

function usage {
  [[ ${1:-} == '-s' ]] || echo "Скрипт установки PromUC"
  echo "Использование:"
  echo "  install.sh [-h|-rfqpc]|--[help|[recreate][force|quiet][pull][continue]]"
  echo "    -h, --help     - отобразить справку"
  echo "    -r, --recreate - переустановить с очисткой данных"
  echo "    -f, --force    - не запрашивать подтверждений действий"
  echo "    -q, --quiet    - см. -f, --force"
  echo "    -p, --pull     - предварительно скачивать дистрибутивы сервисов"
  echo "    -c, --continue - продолжить прерванную установку"
  exit 1
}

forwardRequest() {
  if [[ -z ${ARG_FORCE:-} ]]; then
    IFS= read -erp "Продолжаем? (Y/n): "
    [[ "${REPLY:-y}" =~ ^[yY]$ ]] || exit 1
  fi
}

function prepare_check {
  [[ $(dpkg -l | wc -l) -gt 0 ]] || { echo "Данный дистрибутив не предназначен для ОС $(sed -nr 's/^NAME="(.+)"/\1/p' /etc/os-release)"; exit 1; }
    osname=$(sed -nr 's/^PRETTY_NAME="?(.+)"?/\1/p' /etc/os-release)
    id=$(sed -nr 's/^ID=(.+)/\1/p' /etc/os-release)
    vid=$(sed -nr 's/^VERSION_ID="?([0-9]+)"?/\1/p' /etc/os-release)
    idl=$(sed -nr 's/^ID_LIKE=(.+)/\1/p' /etc/os-release)
  if [[ $id = "debian" ]]; then
    [[ vid -gt 9 ]] || { echo "Работа с ОС $osname не тестировалась и может работать некорректно"; forwardRequest; }
  elif [[ $idl = "debian" ]]; then
    debv=$(head -n1 /etc/debian_version) || { echo "Работа с ОС $osname не поддерживается"; exit 1; }
    case $debv in
      9*) lsbrelease="stretch"; echo "Работа с ОС $osname не тестировалась и может работать некорректно"; forwardRequest ;;
      stretch*) lsbrelease="stretch"; echo "Работа с ОС $osname не тестировалась и может работать некорректно"; forwardRequest ;;
      10*) lsbrelease="buster" ;;
      buster*) lsbrelease="buster" ;;
      11*) lsbrelease="bullseye" ;;
      bullseye*) lsbrelease="bullseye" ;;
      *) echo "Работа с ОС $osname не тестировалась и может работать некорректно"; forwardRequest ;;
    esac
    [[ $id = "ubuntu" ]] && { lsbrelease=""; os="ubuntu"; }
  else
    echo "Работа с ОС $osname не тестировалась и может работать некорректно"; forwardRequest
  fi

  echo "Обновляем кеш пакетной базы..."
  if { apt-get update 2>&1 || echo E: update failed; } | grep -q '^[WE]:'; then
    echo "Ошибка обновления, возможны ошибки при установке необходимых пакетов" && forwardRequest
  fi
  echo "Устанавливаем необходимые дополнительные пакеты curl jq rsync ca-certificates gnupg lsb-release git bzip2 apt-utils apache2-utils"
  { apt-get install -qq apt-utils 1> /dev/null && apt-get install -qq curl jq rsync ca-certificates gnupg lsb-release git bzip2 apache2-utils 1> /dev/null; } || { echo "Ошибка установки"; exit 1; }
  echo -e "\nПроверяем наличие и версии Docker и Compose..."
    if [ -n "$(which docker)" ]; then
    needV=19.03.0
    usedV=$(docker -v | sed -nr 's/Docker version (.+),.*/\1/p')
    [ "$needV" = "$(echo -e "$needV\n$usedV" | sort -V | head -n1)" ] || { echo "Используемая версия Docker $usedV ниже необходимой 19.03.0, требуется обновление."; needUpd=true; }
    if  docker --help | grep -q compose; then
      needV=2.7.0
      usedV=$(docker compose version | sed -nr 's/Docker Compose version v(.+).*/\1/p')
      [ "$needV" = "$(echo -e "$needV\n$usedV" | sort -V | head -n1)" ] || { echo "Используемая версия Docker Compose $usedV ниже необходимой 2.7.0, требуется обновление."; needUpd=true; }
    else
      echo "Требуется установка Compose V2"
      needUpd=true
    fi
  else
    echo "Требуется установка Docker"
    needUpd=true
  fi

  if [[ ${needUpd:-false} = false ]]; then
    echo "Docker Engine и Compose v2 установлены необходимых версий, продолжаем..."
  else
    echo "Требуется провести обновление согласно инструкции https://docs.docker.com/engine/install/${os:-debian}/" && forwardRequest
    echo "Удаляем docker docker-engine docker.io docker-compose containerd runc"
    apt-get -qq remove \
      "$(dpkg -l docker &> /dev/null && echo $_)" \
      "$(dpkg -l docker-engine &> /dev/null && echo $_)" \
      "$(dpkg -l docker.io &> /dev/null && echo $_)" \
      "$(dpkg -l docker-compose &> /dev/null && echo $_)" \
      "$(dpkg -l containerd &> /dev/null && echo $_)" \
      "$(dpkg -l runc &> /dev/null && echo $_)" || { echo "Ошибка при удалении"; forwardRequest; }
    echo "Добавляем репозиторий Docker"
    mkdir -p /etc/apt/keyrings
    curl -fsSL https://download.docker.com/linux/debian/gpg | gpg --dearmor -o /etc/apt/keyrings/docker.gpg
    echo "deb [arch=$(dpkg --print-architecture) signed-by=/etc/apt/keyrings/docker.gpg] https://download.docker.com/linux/${os:-debian} ${lsbrelease:-$(lsb_release -cs)} stable" | tee /etc/apt/sources.list.d/docker.list > /dev/null
    chmod a+r /etc/apt/keyrings/docker.gpg
    if { apt-get update 2>&1 || echo E: update failed; } | grep -q '^[WE]:'; then
      echo "Ошибка обновления пакетной базы, возможны ошибки при установке необходимых пакетов" && forwardRequest
    fi
    echo "Выполняем установку docker-ce docker-ce-cli containerd.io docker-compose-plugin"
    apt-get install -qq docker-ce docker-ce-cli containerd.io docker-compose-plugin 1> /dev/null || { echo "Ошибка при установке"; exit 1; }
    cp install/docker/daemon.json /etc/docker/
    while true; do
      systemctl restart docker && break
      IFS= read -erp "Повторить попытку запуска Docker? [Y/n]: "
      [[ "${REPLY:-y}" =~ ^[yY]$ ]] && continue || exit 1
    done
  fi

  [ ! -f .order ] && cp template.order .order
  [[ -z ${ARG_FORCE:-} ]] && { echo -e "Согласно файла .order будут установлены следующие компоненты:\n\n$(grep -v '^ *#' .order)\n\nЕсли необходимо скорректировать то нажмите 'n', отредактируйте данный файл и повторно запустите установку"; forwardRequest; }

  if [[ $(docker network ls -f name=proxy_backend -q | wc -l) -gt 0 ]]; then
    if [[ -n $(docker network inspect proxy_backend | grep Name | tail -n +2) ]]; then
      echo -e "\nУдаляем контейнеры предыдущей установки..."
      "$installdir"/do.sh -r --all down -v
    fi
  fi
}

# DEBUG options
if [[ -n ${DEBUG:-} ]]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

PROGRAM=${0##*/}
workdir=$(dirname $(realpath -s $0))
installdir=$workdir
cd "$workdir" || exit 1

source "$installdir"/install/functions.sh &> /dev/null || { echo -e "FAIL\nОшибка подключения библиотеки функций из файла $installdir/install/functions.sh"; exit 1; }

while getopts ':hrfqpc-:' KEY; do
  case $KEY in
    h )     usage -s ;;
    r )     ARG_RECREATE="-$KEY" ;;
    f | q ) ARG_FORCE="-$KEY" ;;
    p )     ARG_PULL="-$KEY" ;;
    c )     ARG_CONTINUE="-$KEY" ;;
    - )
        case $OPTARG in
          help )     usage ;;
          recreate ) ARG_RECREATE="--$OPTARG" ;;
          force | quiet ) ARG_FORCE="--$OPTARG" ;;
          pull )     ARG_PULL="--$OPTARG" ;;
          continue ) ARG_CONTINUE="--$OPTARG" ;;
          * )
              echo "Не верный ключ '--$OPTARG' при вызове $PROGRAM"
              usage -s
          ;;
        esac
    ;;
    * )
        echo "Не верный ключ '-$OPTARG' при вызове $PROGRAM"
        usage -s
    ;;
  esac
done

if [[ -z ${ARG_FORCE:-} ]]; then
  if [[ -z ${ARG_CONTINUE:-} ]]; then
    IFS= read -erp "Выполнить установку PromUC (Y/n): "
  else
    IFS= read -erp "Продолжить установку PromUC (Y/n): "
  fi
  [[ "${REPLY:-y}" =~ ^[yY]$ ]] || exit 1
fi

[[ -z ${ARG_CONTINUE:-} ]] && prepare_check

# from install/functions.sh
create_config

# TODO re-work
# shellcheck disable=SC2013
readarray -t services < <(grep -v '^ *#' "$installdir"/.order)
for ((i=0; i<${#services[@]}; i++)); do
  service=${services[$i]}
  [[ -d $installdir/$service ]] || { echo -e "\nПропущен сервис $service, проверьте наличие каталога"; continue; }
  if [[ $ARG_CONTINUE ]] && [[ -f $installdir/${services[$i]}/.env ]]; then
    continue
  fi
  # shellcheck disable=SC2086
  "$installdir"/"${services[$i]}"/install.sh ${ARG_RECREATE:-} 
  if [[ -z ${ARG_FORCE:-} ]]; then
    next_service=${services[$i+1]}
    if [[ -n $next_service ]]; then
      echo "Следующим будет установлен сервсис $next_service"
      forwardRequest
    fi
  fi
done

echo -e "\nУстановка Promuc окончена\n"
cd "$installdir" || exit 1
[ -f .config ] && { echo -e "\nОсновные реквизиты доступа в файле $workdir/.config\n"; cat .config; }
