#!/bin/bash

# DEBUG options
if [ ! -z ${DEBUG:-} ]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

set -euo pipefail

workdir=$(dirname $(realpath -s $0))
installdir=$(dirname $workdir)

cd $installdir/proxy/conf || exit 1

echo "Обновляем сертификат CA..."
source $installdir/.env
cp certs/ca.crt certs/ca.crt.old
openssl req -new -x509 -config ca.cnf -batch -key ca/ca.key -out certs/ca.crt -subj "/CN=$SERVICE_DOMAIN" -days 3650
rm -f /usr/local/share/ca-certificates/ca.crt
rm -f /usr/local/share/ca-certificates/ca-$SERVICE_DOMAIN.crt
update-ca-certificates &> /dev/null || { echo "Ошибка выполнения update-ca-certificates"; exit 1; }
cp certs/ca.crt /usr/local/share/ca-certificates/ca-$SERVICE_DOMAIN.crt
update-ca-certificates || { echo "Ошибка выполнения update-ca-certificates"; exit 1; }

echo "Обновляем сервис Updater"
cp certs/ca-certificates.crt certs/ca.crt $installdir/updater/
$installdir/do.sh updater up -d --force-recreate

echo "Обновляем сервис Scada"
cp certs/ca.crt $installdir/scada/
$installdir/do.sh scada up -d --force-recreate

echo -e "\nОбновление сертификата выполнено. Старый сохранён в $installdir/proxy/conf/certs/ca.crt.old\n"
