#!/bin/bash 

# Функция проверки и установки сервиса доменов в файлах .env
function set_service_domain {
  while ! grep -qP '(?=^.{4,253}$)(^(?:[a-zA-Z0-9](?:(?:[a-zA-Z0-9\-]){0,61}[a-zA-Z0-9])?\.)+([a-zA-Z]{2,}|xn--[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])$)' <<< "$SERVICE_DOMAIN"; do
    read -erp "Введите домен сервисов 2-го уровня или более: " -i "${SERVICE_DOMAIN:-promuc.local}" SERVICE_DOMAIN
  done
  [[ -f "$installdir"/.env ]] || cp "$installdir"/template.env "$installdir"/.env
  sed -i -r "s/(SERVICE_DOMAIN=).*/\1$SERVICE_DOMAIN/g" "$installdir"/.env
  sed -i -r "s/(SERVICE_DOMAIN=).*/\1$SERVICE_DOMAIN/g" .env
}

function set_project {
  while ! grep -qP '^(?:[a-zA-Z0-9](?:[a-zA-Z0-9_-]))$' <<< "$PROJECT_NAME"; do
    read -erp "Введите название проекта: " -i "${PROJECT_NAME:-}" PROJECT_NAME
  done
  [[ ! -f "$installdir"/.env ]] || cp "$installdir"/template.env "$installdir"/.env
  sed -i -r "s/(PROJECT_NAME=).*/\1$PROJECT_NAME/g" "$installdir"/.env
  sed -i -r "s/(PROJECT_NAME=).*/\1$PROJECT_NAME/g" .env
}

function host_add {
  site="$SERVICE_DOMAIN"
  [[ -n ${1:-} ]] && site="$1.$SERVICE_DOMAIN"
  if [[ -z $(getent hosts "$site" | awk '{ print $1 }') ]]; then
   echo -e "\nНе найден адрес $site, добавляем в /etc/hosts\n"
   echo -e "127.0.0.1\t$site # for PromUC" >> /etc/hosts
  else
   sed -i -r "s/^[0-9\.]+(\s+$site.*$)/127.0.0.1\1/g" /etc/hosts
  fi
}

function parse_yaml {
   local prefix=$2
   local separator=${3:-_}

   local indexfix
   # Detect awk flavor
   if awk --version 2>&1 | grep -q "GNU Awk" ; then
      # GNU Awk detected
      indexfix=-1
   elif awk -Wv 2>&1 | grep -q "mawk" ; then
      # mawk detected
      indexfix=0
   fi

   local s='[[:space:]]*' sm='[ \t]*' w='[a-zA-Z0-9_]*' fs=${fs:-$(echo @|tr @ '\034')} i=${i:-  }
   cat $1 | \
   awk -F$fs "{multi=0; 
       if(match(\$0,/$sm\|$sm$/)){multi=1; sub(/$sm\|$sm$/,\"\");}
       if(match(\$0,/$sm>$sm$/)){multi=2; sub(/$sm>$sm$/,\"\");}
       while(multi>0){
           str=\$0; gsub(/^$sm/,\"\", str);
           indent=index(\$0,str);
           indentstr=substr(\$0, 0, indent+$indexfix) \"$i\";
           obuf=\$0;
           getline;
           while(index(\$0,indentstr)){
               obuf=obuf substr(\$0, length(indentstr)+1);
               if (multi==1){obuf=obuf \"\\\\n\";}
               if (multi==2){
                   if(match(\$0,/^$sm$/))
                       obuf=obuf \"\\\\n\";
                       else obuf=obuf \" \";
               }
               getline;
           }
           sub(/$sm$/,\"\",obuf);
           print obuf;
           multi=0;
           if(match(\$0,/$sm\|$sm$/)){multi=1; sub(/$sm\|$sm$/,\"\");}
           if(match(\$0,/$sm>$sm$/)){multi=2; sub(/$sm>$sm$/,\"\");}
       }
   print}" | \
   sed  -e "s|^\($s\)?|\1-|" \
       -ne "s|^$s#.*||;s|$s#[^\"']*$||;s|^\([^\"'#]*\)#.*|\1|;t1;t;:1;s|^$s\$||;t2;p;:2;d" | \
   sed -ne "s|,$s\]$s\$|]|" \
        -e ":1;s|^\($s\)\($w\)$s:$s\(&$w\)\?$s\[$s\(.*\)$s,$s\(.*\)$s\]|\1\2: \3[\4]\n\1$i- \5|;t1" \
        -e "s|^\($s\)\($w\)$s:$s\(&$w\)\?$s\[$s\(.*\)$s\]|\1\2: \3\n\1$i- \4|;" \
        -e ":2;s|^\($s\)-$s\[$s\(.*\)$s,$s\(.*\)$s\]|\1- [\2]\n\1$i- \3|;t2" \
        -e "s|^\($s\)-$s\[$s\(.*\)$s\]|\1-\n\1$i- \2|;p" | \
   sed -ne "s|,$s}$s\$|}|" \
        -e ":1;s|^\($s\)-$s{$s\(.*\)$s,$s\($w\)$s:$s\(.*\)$s}|\1- {\2}\n\1$i\3: \4|;t1" \
        -e "s|^\($s\)-$s{$s\(.*\)$s}|\1-\n\1$i\2|;" \
        -e ":2;s|^\($s\)\($w\)$s:$s\(&$w\)\?$s{$s\(.*\)$s,$s\($w\)$s:$s\(.*\)$s}|\1\2: \3 {\4}\n\1$i\5: \6|;t2" \
        -e "s|^\($s\)\($w\)$s:$s\(&$w\)\?$s{$s\(.*\)$s}|\1\2: \3\n\1$i\4|;p" | \
   sed  -e "s|^\($s\)\($w\)$s:$s\(&$w\)\(.*\)|\1\2:\4\n\3|" \
        -e "s|^\($s\)-$s\(&$w\)\(.*\)|\1- \3\n\2|" | \
   sed -ne "s|^\($s\):|\1|" \
        -e "s|^\($s\)\(---\)\($s\)||" \
        -e "s|^\($s\)\(\.\.\.\)\($s\)||" \
        -e "s|^\($s\)-$s[\"']\(.*\)[\"']$s\$|\1$fs$fs\2|p;t" \
        -e "s|^\($s\)\($w\)$s:$s[\"']\(.*\)[\"']$s\$|\1$fs\2$fs\3|p;t" \
        -e "s|^\($s\)-$s\(.*\)$s\$|\1$fs$fs\2|" \
        -e "s|^\($s\)\($w\)$s:$s[\"']\?\(.*\)$s\$|\1$fs\2$fs\3|" \
        -e "s|^\($s\)[\"']\?\([^&][^$fs]\+\)[\"']$s\$|\1$fs$fs$fs\2|" \
        -e "s|^\($s\)[\"']\?\([^&][^$fs]\+\)$s\$|\1$fs$fs$fs\2|" \
        -e "s|$s\$||p" | \
   awk -F$fs "{
      gsub(/\t/,\"        \",\$1);
      if(NF>3){if(value!=\"\"){value = value \" \";}value = value  \$4;}
      else {
        if(match(\$1,/^&/)){anchor[substr(\$1,2)]=full_vn;getline};
        indent = length(\$1)/length(\"$i\");
        vname[indent] = \$2;
        value= \$3;
        for (i in vname) {if (i > indent) {delete vname[i]; idx[i]=0}}
        if(length(\$2)== 0){  vname[indent]= ++idx[indent] };
        vn=\"\"; for (i=0; i<indent; i++) { vn=(vn)(vname[i])(\"$separator\")}
        vn=\"$prefix\" vn;
        full_vn=vn vname[indent];
        if(vn==\"$prefix\")vn=\"$prefix$separator\";
        if(vn==\"_\")vn=\"__\";
      }
      assignment[full_vn]=value;
      if(!match(assignment[vn], full_vn))assignment[vn]=assignment[vn] \" \" full_vn;
      if(match(value,/^\*/)){
         ref=anchor[substr(value,2)];
         if(length(ref)==0){
           printf(\"%s=\\\"%s\\\"\n\", full_vn, value);
         } else {
           for(val in assignment){
              if((length(ref)>0)&&index(val, ref)==1){
                 tmpval=assignment[val];
                 sub(ref,full_vn,val);
                 if(match(val,\"$separator\$\")){
                    gsub(ref,full_vn,tmpval);
                 } else if (length(tmpval) > 0) {
                    printf(\"%s=\\\"%s\\\"\n\", val, tmpval);
                 }
                 assignment[val]=tmpval;
              }
           }
         }
      } else if (length(value) > 0) {
         printf(\"%s=\\\"%s\\\"\n\", full_vn, value);
      }
   }END{
      for(val in assignment){
         if(match(val,\"$separator\$\"))
            printf(\"%s=\\\"%s\\\"\n\", val, assignment[val]);
      }
   }"
}

function create_config {
  cat > "$installdir"/install/config.json <<EOF
{
   "plugins":[
   
   ]
}
EOF
}

function create_storage_volume {
  while :; do
    echo -n "Создаём docker volume promuc_storage..."
    docker volume create promuc_storage 1> /dev/null && { echo "OK"; break; }
    echo "FAIL"
    read -erp "Не удалось создать docker volume. Повторить? (Y/n): "
    [[ "${REPLY:-y}" =~ ^[yY]$ ]] || exit 2
  done
}

function stop_storage_volume {
  # останавливаем контейнеры использующие хранилище (кеш) репозиториев
  volume_containers=()
  storage_volume=( $(docker volume ls -qf name=promuc_storage) )
  if [[ ${#storage_volume[@]} -gt 0 ]] && [[ $storage_volume = promuc_storage ]]; then
    readarray volume_containers -t < <(docker ps -f volume=promuc_storage --format '{{.Names}} {{.Label "com.docker.compose.project.config_files"}} {{.Label "com.docker.compose.service"}}')
    for container in "${volume_containers[@]}"; do
      container_params=( $container )
      while :; do
        echo "Останавливаем и удаляем контейнер ${container_params[0]}..."
        docker compose -f ${container_params[1]} rm -sfv ${container_params[2]} &&  break
        read -erp "Не удалось удалить контейнер. Повторить? (Y/n): "
        [[ "${REPLY:-y}" =~ ^[yY]$ ]] || exit 2    
      done
    done
    # удаляем хранилище (кеш) репозиториев
    while :; do
      echo -n "Удаляем docker volume promuc_storage..."
      docker volume rm $storage_volume 1> /dev/null && { echo "OK"; break; }
      echo "FAIL"
      read -erp "Не удалось удалить docker volume. Повторить? (Y/n): "
      [[ "${REPLY:-y}" =~ ^[yY]$ ]] || exit 2
    done
  fi
  # создаём volume хранилища репозиториев
  create_storage_volume
  # запускаем ранее остановленные контейнеры
  for container in "${volume_containers[@]}"; do
    container_params=( $container )
    while :; do
      echo "Запускаем контейнер ${container_params[0]}..."
      docker compose -f ${container_params[1]} up -d --wait ${container_params[2]} && break
      read -erp "Не удалось запустить контейнер. Повторить? (Y/n): "
      [[ "${REPLY:-y}" =~ ^[yY]$ ]] || exit 2
    done
  done
}

function check_storage_volume {
  storage_volume=( $(docker volume ls -qf name=promuc_storage) )
  if [[ ${#storage_volume[@]} -eq 0 ]] || [[ ! $storage_volume = promuc_storage ]]; then
    create_storage_volume
  fi
}