##########################################################################
# External Authentication Sources
##########################################################################

# Default setting is internal
# External Supported Sources: ldap, kerberos, oauth2
# Multiple authentication can be achieved by setting this parameter to
# ['ldap', 'internal'] or ['oauth2', 'internal'] or
# ['webserver', 'internal'] etc.
# pgAdmin will authenticate the user with ldap/oauth2 whatever first in the
# list, in case of failure the second authentication option will be considered.

AUTHENTICATION_SOURCES = ['oauth2','internal']

##########################################################################
# OAuth2 Configuration
##########################################################################

# Multiple OAUTH2 providers can be added in the list like [{...},{...}]
# All parameters are required

OAUTH2_CONFIG = [
    {
        # The name of the of the oauth provider, ex: github, google
        'OAUTH2_NAME': 'gitea',
        # The display name, ex: Google
        'OAUTH2_DISPLAY_NAME': 'Gitea',
        # Oauth client id
        'OAUTH2_CLIENT_ID': 'V_OAUTH2_CLIENT_ID',
        # Oauth secret
        'OAUTH2_CLIENT_SECRET': 'V_OAUTH2_CLIENT_SECRET',
        # URL to generate a token,
        # Ex: https://github.com/login/oauth/access_token
        'OAUTH2_TOKEN_URL': 'https://git.SERVICE_DOMAIN/login/oauth/access_token',
        # URL is used for authentication,
        # Ex: https://github.com/login/oauth/authorize
        'OAUTH2_AUTHORIZATION_URL': 'https://git.SERVICE_DOMAIN/login/oauth/authorize',
        # server metadata url might optional for your provider
        'OAUTH2_SERVER_METADATA_URL': 'https://git.SERVICE_DOMAIN/.well-known/openid-configuration',
        # Oauth base url, ex: https://api.github.com/
        'OAUTH2_API_BASE_URL': 'https://git.SERVICE_DOMAIN/api/v1',
        # Name of the Endpoint, ex: user
        'OAUTH2_USERINFO_ENDPOINT': 'https://git.SERVICE_DOMAIN/login/oauth/userinfo',
        # Oauth scope, ex: 'openid email profile'
        # Note that an 'email' claim is required in the resulting profile
        'OAUTH2_SCOPE': 'openid user email profile',
        # The claim which is used for the username. If the value is empty the
        # email is used as username, but if a value is provided,
        # the claim has to exist.
        'OAUTH2_USERNAME_CLAIM': 'email',
        # Font-awesome icon, ex: fa-github
        'OAUTH2_ICON': 'fa-gitea',
        # UI button colour, ex: #0000ff
        'OAUTH2_BUTTON_COLOR': '#528321',
    }
]

# After Oauth authentication, user will be added into the SQLite database
# automatically, if set to True.
# Set it to False, if user should not be added automatically,
# in this case Admin has to add the user manually in the SQLite database.

OAUTH2_AUTO_CREATE_USER = True

##########################################################################
# Master password is used to encrypt/decrypt saved server passwords
# Applicable for desktop mode only
##########################################################################
MASTER_PASSWORD_REQUIRED = True
