#!/bin/bash

# DEBUG options
if [ ! -z ${DEBUG:-} ]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

set -euo pipefail

echo -e "\nНастраиваем pgadmin\n"

workdir=$(dirname $(realpath -s $0))
installdir=$(dirname $workdir)
source "$installdir"/install/functions.sh &> /dev/null || { echo -e "FAIL\nОшибка подключения библиотеки функций из файла $installdir/install/functions.sh"; exit 1; }
cd $workdir || exit 1


if [ ! -f .env ] || [[ ${1:-} = "--recreate" ]] || [[ ${1:-} = "-r" ]]; then
  [ ! -f docker-compose.yml ] && cp docker-compose-cert.yml docker-compose.yml
  [ $(docker compose ps -aq | wc -l) -gt 0 ] && docker compose down -v

  cp template.env .env
  source .env
  [ -f ../.env ] && source ../.env

  # from install/functions.sh
  set_service_domain
  host_add pgadmin

  sed -i -r "s/(PGADMIN_DEFAULT_EMAIL=).*/\1$(sed -n 's|email:  ||p' ../.config)/" .env
  sed -i -r "s/(PGADMIN_DEFAULT_PASSWORD=).*/\1$(sed -n "s|TRAEFIK_ADMIN_PASS=||p" ../proxy/.env | tr -d '\n')/" .env
  source .env
  echo -e "Доступ к PgAdmin4\n\n\tURL: https://pgadmin.$SERVICE_DOMAIN\n\tлогин: $PGADMIN_DEFAULT_EMAIL\n\tпароль: $PGADMIN_DEFAULT_PASSWORD"

  echo -e "\n### PgAdmin ###" >> ../.config
  echo "логин:  $PGADMIN_DEFAULT_EMAIL" >> ../.config
  echo "пароль: $PGADMIN_DEFAULT_PASSWORD" >> ../.config

  rm -rf ./.secrets && mkdir ./.secrets
  
  source $installdir/proxy/.env
  case ${CERT:-1} in
    2 ) cp docker-compose-le.yml docker-compose.yml ;;
    * ) cp docker-compose-cert.yml docker-compose.yml ;;
  esac
  # настройка аутентификации в Gitea
  if [ "$(grep -xc "gitea" $installdir/.order)" -gt 0 ]; then 
    while true; do
      if [ $(docker ps -q -f name=gitea | wc -l) -gt 0 ]; then
        if ! source $workdir/giteauth.sh; then
          read -r -p "Повторить настройку аутентификации? [Y/n]: " ANSWER
          [[ "${ANSWER:-y}" =~ ^[yY]$ ]] && continue || { echo "Не удалось настроить аутентификацию OAuth. Для настройки запустите скрипт $installdir/pgadmin/giteauth.sh"; break; }
        fi
      else
        read -r -p "Для настройки аутентификации Oauth необходимо запустить сервис Gitea. Запустить? [Y/n]: " ANSWER
        if [[ "${ANSWER:-y}" =~ ^[yY]$ ]]; then
          cd $installdir/gitea
          docker compose up -d && continue || { echo "Не удалось запустить Gitea"; exit 1; }
        fi
      fi
      break
    done
    cd $workdir
    
    rm -rf ./.secrets && mkdir ./.secrets
    echo $PGADMIN_DEFAULT_PASSWORD > ./.secrets/pgpass
  fi
fi

source $installdir/proxy/.env
if [[ ! "$CERT" == "2" ]]; then 
  [ -d ca.crt ] && rm -rf ca.crt
  if [ -f $installdir/proxy/conf/certs/ca.crt ]; then
    cp -p $installdir/proxy/conf/certs/ca.crt $workdir
  else
    echo -e "Отсутствует файл корневого сертификата ca.crt в каталоге прокси\nВозможно прокси не был установлен или произошла ошибка при его запуске, повторите его настройку\nНастройка прервана\n"
    exit 1
  fi
  [ -d ca-certificates.crt ] && rm -rf ca-certificates.crt
  if [ -f $installdir/proxy/conf/certs/ca-certificates.crt ]; then
    cp -p $installdir/proxy/conf/certs/ca-certificates.crt $workdir
  else
    echo -e "Отсутствует файл сертификатов ca-certificates.crt в каталоге прокси\nВозможно прокси не был установлен или произошла ошибка при его запуске, повторите его настройку\nНастройка прервана\n"
    exit 1
  fi
fi

if [[ $(grep '#- pgadmin.${SERVICE_DOMAIN:-local}' $installdir/proxy/docker-compose.yml | wc -l) -gt 0 ]]; then
  sed -i "s/#- pgadmin/- pgadmin/" $installdir/proxy/docker-compose.yml
  cd $installdir/proxy || exit 1
  if [ $(docker compose ps -aq | wc -l) -gt 0 ]; then
    echo "Перезапускаем сервис proxy..."
    success=1
    while [ ${success:-1} -eq 1 ]; do
      docker compose up -d --force-recreate 1> /dev/null && success=0 || { echo "Ошибка рестарта"; success=1; }
      if [ $success -ne 0 ]; then
        read -r -p "Повторить попытку? [Y/n]: " ANSWER
        [[ "${ANSWER:-y}" =~ ^[yY]$ ]] || exit 1
      fi
    done
  fi
fi
cd $workdir
echo -e "\nЗапускаем pgadmin\n"
[ $(docker image ls -q dpage/pgadmin4:${PGADMIN_TAG:-6.19} | wc -l) -eq 0 ] && { $installdir/getimage.sh dpage/pgadmin4:${PGADMIN_TAG:-6.19} || exit 1; }
docker compose up -d --force-recreate --wait || { echo "Ошибка при запуске контейнера. Повторите установку."; exit 2; }


