#!/bin/bash

# DEBUG options
if [ ! -z ${DEBUG:-} ]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

set -euo pipefail

echo -e "\nНастраиваем portainer\n"

workdir=$(dirname $(realpath -s $0))
installdir=$(dirname $workdir)
source "$installdir"/install/functions.sh &> /dev/null || { echo -e "FAIL\nОшибка подключения библиотеки функций из файла $installdir/install/functions.sh"; exit 1; }
cd $workdir || exit 1

[ -f .env ] || cp template.env .env
source .env
[ $(docker image ls -q portainer/portainer-ce:${PORTAINER_TAG:-latest} | wc -l) -eq 0 ] && { $installdir/getimage.sh portainer/portainer-ce:${PORTAINER_TAG:-latest} || exit 1; }

if [ ! -f .env ] || [[ ${1:-} = "--recreate" ]] || [[ ${1:-} = "-r" ]]; then
  [ ! -f docker-compose.yml ] && cp docker-compose-cert.yml docker-compose.yml
  [ $(docker compose ps -aq | wc -l) -gt 0 ] && docker compose down -v

  cp template.env .env
  source .env
  [ -f ../.env ] && source ../.env

  # from install/functions.sh
  set_service_domain
  host_add portainer

  [[ -f $installdir/proxy/.env ]] && source "$installdir"/proxy/.env
  case ${CERT:-1} in
    2 ) cp docker-compose-le.yml docker-compose.yml ;;
    * ) cp docker-compose-cert.yml docker-compose.yml ;;
  esac

  echo -e "\nЗапускаем portainer"
  docker compose up -d --force-recreate --wait || { echo "Ошибка при запуске контейнера. Повторите установку."; exit 2; }
  
  # настройка администратора
  echo -en "\nОжидаем доступность API Portainer..."
  timer=0
  while [ $timer -lt 60 ]; do
    (( timer+=1 ))
    responseCode=$(\
      curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X GET \
      https://portainer.$SERVICE_DOMAIN/api/system/status \
      -H 'accept: application/json'
    ) && { [[ "$responseCode" = 200 ]] && { echo "OK"; break; }; }
    [ $timer -eq 60 ] || { sleep 1; continue; }
    echo "FAIL"
    read -r -p "API Portainer не доступно в течении минуты, повторить запрос? (Y/n): " ANSWER
    [[ "${ANSWER:-y}" =~ ^[yY]$ ]] && timer=0 || exit 1
  done
  # получаем логин администратора
  if [[ -z ${TRAEFIK_ADMIN_LOGIN:-} ]]; then
    IFS= read -erp "Введите логин администратора: " -i "${TRAEFIK_ADMIN_LOGIN:-admin}"
    TRAEFIK_ADMIN_LOGIN=${REPLY:-admin}
  fi
  # получаем пароль администратора
  if [[ -z ${TRAEFIK_ADMIN_PASS:-} ]]; then
    TRAEFIK_ADMIN_PASS=$(date | md5sum | head -c 12)
    while :; do
      IFS= read  -erp "Введите пароль администратора (не менее 12 символов): " -i "$TRAEFIK_ADMIN_PASS" TRAEFIK_ADMIN_PASS
      [[ ${#TRAEFIK_ADMIN_PASS} -lt 12 ]] && { echo "Пароль менее 12 символов"; continue; } || break
    done
  fi
  echo -n "Настраиваем администратора..."
  curl -s --noproxy "*" -X POST \
    https://portainer.${SERVICE_DOMAIN:-localhost}/api/users/admin/init \
      -H "Content-Type: application/json" \
      -d '{
            "Username":"'"$TRAEFIK_ADMIN_LOGIN"'",
            "Password":"'"$TRAEFIK_ADMIN_PASS"'"
      }' &>/dev/null || echo "FAIL"
  echo "OK"
  
  echo -e "Доступ к Portainer\n\n\tURL: https://portainer.$SERVICE_DOMAIN\n\tлогин: $TRAEFIK_ADMIN_LOGIN\n\tпароль: $TRAEFIK_ADMIN_PASS"
  echo -e "\n### Portainer ###" >> ../.config
  echo "логин:  $TRAEFIK_ADMIN_LOGIN" >> ../.config
  echo "пароль: $TRAEFIK_ADMIN_PASS" >> ../.config
else
  echo -e "\nЗапускаем portainer"
  docker compose up -d --force-recreate --wait || { echo "Ошибка при запуске контейнера. Повторите установку."; exit 2; }
fi