#!/bin/bash

# DEBUG options
if [ ! -z ${DEBUG:-} ]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

set -euo pipefail

workdir=$(dirname $(realpath -s $0))
installdir=$(dirname $workdir)
cd $workdir || exit 1

echo -e "\nНастраиваем postgres\n"

if [ ! -d data ] || [[ ${1:-} = "--recreate" ]] || [[ ${1:-} = "-r" ]]; then
  [ $(docker compose ps -aq | wc -l) -gt 0 ] && docker compose down -v
  [ -f ../.env ] && source ../.env
  cp template.env .env
  rm -rf data/
  mkdir -p data
  chown -R 999:999 data/
  
  [ -d .secrets ] || mkdir .secrets
  sed -n "s/POSTGRES_PASSWORD=//p" .env | tr -d \n > .secrets/postgres-password

  echo -e "Запускаем postgres"
  [ $(docker image ls -q postgres:${POSTGRES_TAG:-14} | wc -l) -eq 0 ] && { $installdir/getimage.sh postgres:${POSTGRES_TAG:-14} || exit 1; }
  docker compose up -d --force-recreate --wait || { echo "Ошибка при запуске контейнера. Повторите установку."; exit 2; }
  echo -n "Ожидаем готовность postgres..."
  for ((timer=0; timer < 61; timer++)); do
    docker exec -u postgres postgres pg_ctl status | grep "pg_ctl: server is running (PID: 1)" &> /dev/null && { echo "OK"; break; }
    [ $timer -eq 60 ] || { sleep 1; continue; }
    echo -e "\n"
    read -r -p "Готовность postgres не подтверждена в течении минуты. Повторить запрос? [Y/n]: " ANSWER
    [[ "${ANSWER:-y}" =~ ^[yY]$ ]] &&  timer=0 || exit 1
  done
else
  echo -e "Возможно имеются данные, установка игнорируется.\nДля принудительной установки используйте флаг --recreate\n"
fi
