#!/bin/bash

# DEBUG options
if [ ! -z ${DEBUG:-} ]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

set -euo pipefail

genCert() {
  sed -i "s/CERT=.*/CERT=1/g" ../.env
  rm -rf ca db certs certs.cnf
  mkdir ca db certs

  echo -n "Генерируем сертификаты..."

  openssl genrsa -out ca/ca.key 2048
  openssl req -new -x509 -config ca.cnf -batch -key ca/ca.key -out certs/ca.crt -subj "/CN=$SERVICE_DOMAIN" -days 3650
  rm -f /usr/local/share/ca-certificates/ca.crt
  rm -f /usr/local/share/ca-certificates/ca-$SERVICE_DOMAIN.crt
  update-ca-certificates &> /dev/null || { echo "Ошибка выполнения update-ca-certificates"; exit 1; }
  cp certs/ca.crt /usr/local/share/ca-certificates/ca-$SERVICE_DOMAIN.crt
  update-ca-certificates &> /dev/null || { echo "Ошибка выполнения update-ca-certificates"; exit 1; }

  touch db/index.txt
  echo $(date +%s) > db/serial.txt
  echo "unique_subject = no" > db/index.txt.attr
  openssl genrsa -out certs/$SERVICE_DOMAIN.key 2048

  cp template.certs.cnf certs.cnf
  sed -i "s/domain/$SERVICE_DOMAIN/g" certs.cnf

  openssl req -new -config certs.cnf -batch -key certs/$SERVICE_DOMAIN.key -out certs/$SERVICE_DOMAIN.csr
  openssl ca -config ca.cnf -keyfile ca/ca.key -cert certs/ca.crt -policy signing_policy -extensions signing_node_req -batch -in certs/$SERVICE_DOMAIN.csr -out certs/$SERVICE_DOMAIN.cert -outdir certs/ &>/dev/null

  echo "OK"

  cp certs.yml dynamic/
  sed -i -r "s/(certFile: ).*/\1\/cert\/$SERVICE_DOMAIN.cert/" dynamic/certs.yml
  sed -i -r "s/(keyFile: ).*/\1\/cert\/$SERVICE_DOMAIN.key/" dynamic/certs.yml
}

selectCert() {
  sed -i "s/CERT=.*/CERT=3/g" ../.env
  rm -rf ca db certs certs.cnf
  mkdir certs
  certs=()
  while : ; do
    read -r -p "Укажите расположение файла сертификата корневого центра сертифкации или файла-цепочки с сертификатами корневого и промежуточных сертификатов : " ANSWER
    [ -f "$ANSWER" ] && { certs+=( $ANSWER ); break; } || echo -e "\nФайл не найден"
  done
  while : ; do
    read -r -p "Укажите расположение сертификата выданного корневым центром сертифкации: " ANSWER
    [ -f "$ANSWER" ] && { certs+=( $ANSWER ); break; } || echo -e "\nФайл не найден"
  done
  openssl verify -CAfile "${certs[0]}" "${certs[1]}" &> /dev/null || { echo "Ошибка проверки доверия корневого или цепочки сертификатов и выданного сертификатов"; exit 1; }
  while : ; do
    read -r -p "Укажите расположение ключевого файла выданного сертификата: " ANSWER
    [ -f "$ANSWER" ] && { certs+=( $ANSWER ); break; } || echo -e "\nФайл не найден"
  done  
  openssl rsa -check -noout -in "${certs[2]}" &> /dev/null || { echo "Ошибка проверки ключевого файла";  exit 1; }
  [[ $(openssl x509 -modulus -noout -in "${certs[1]}" | openssl sha256 | cut -d' ' -f2) == $(openssl rsa -modulus -noout -in "${certs[2]}" | openssl sha256 | cut -d' ' -f2) ]] || { echo "Ошибка проверки соответствия сертификата и ключевого файла";  exit 1; }
  
  cp "${certs[0]}" certs/ca.crt
  cp "${certs[1]}" certs/$SERVICE_DOMAIN.cert
  cp "${certs[2]}" certs/$SERVICE_DOMAIN.key
  
  echo -e "\nКопируем корневой сертификат в хранилище доверенных корневых центров"
  rm -f /usr/local/share/ca-certificates/ca.crt
  rm -f /usr/local/share/ca-certificates/ca-$SERVICE_DOMAIN.crt
  update-ca-certificates &> /dev/null || { echo "Ошибка выполнения update-ca-certificates"; exit 1; }
  cp certs/ca.crt /usr/local/share/ca-certificates/ca-$SERVICE_DOMAIN.crt
  update-ca-certificates &> /dev/null || { echo "Ошибка выполнения update-ca-certificates"; exit 1; }
  
  cp certs.yml dynamic/
  sed -i -r "s/(certFile: ).*/\1\/cert\/$SERVICE_DOMAIN.cert/" dynamic/certs.yml
  sed -i -r "s/(keyFile: ).*/\1\/cert\/$SERVICE_DOMAIN.key/" dynamic/certs.yml
}

[ $# -eq 0 ] && { echo "Не передан аргумент"; exit 1; }

workdir=$(dirname $(realpath -s $0))
cd $workdir

[ -f ../.env ] && source ../.env
if [ -z "${SERVICE_DOMAIN:-}" ]; then
  while [ -z $(echo $SERVICE_DOMAIN | grep -P '(?=^.{4,253}$)(^(?:[a-zA-Z0-9](?:(?:[a-zA-Z0-9\-]){0,61}[a-zA-Z0-9])?\.)+([a-zA-Z]{2,}|xn--[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])$)') ]; do
    read -e -p "Введите домен сервисов 2-го уровня или более: " -i "${SERVICE_DOMAIN:-}" SERVICE_DOMAIN
  done
  echo "SERVICE_DOMAIN=$SERVICE_DOMAIN" > ../.env
fi
sed -i -r "s/(SERVICE_DOMAIN=).*/\1$SERVICE_DOMAIN/" .env
source ../.env

cd conf || exit 1

CERT="$1"
if [ $CERT = 3 ]; then
  selectCert
else
  genCert
fi
cp traefik-cert.yml traefik.yml
echo -e "\n\tСертификат CA доступен в $(pwd)/certs/ca.crt и по ссылке https://$SERVICE_DOMAIN/ca.crt\n\tНеобходимо импортировать в браузер и в системный список доверенных корневых центров сертификации"
echo -e "\n### Сертификат CA ###\n\n$(cat certs/ca.crt)" >> ../../.config
cd $workdir
