#!/bin/bash

# DEBUG options
if [[ "${0##*/}" == "chpass.sh" ]] &&  [ "$DEBUG" ]; then
  exec 5> "$(dirname "$(realpath -s "$0")")"/."$(basename "$0")".log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

set -euo pipefail

workdir=$(dirname "$(realpath -s "$0")")
installdir=$(dirname "$workdir")
cd "$workdir" || exit 1

[ -f "$installdir"/.env ] || { echo "Отсутствует файл $installdir/.env"; exit 2; }
[ -f .env ] || { echo "Отсутствует файл $workdir/.env"; exit 2; }
source "$installdir"/.env
source .env


while : ; do
  read -r -e -p "Введите логин администратора: " -i "${TRAEFIK_ADMIN_LOGIN:-admin}" TRAEFIK_ADMIN_LOGIN
  [ -z "$TRAEFIK_ADMIN_LOGIN" ] && continue || break
done
sed -i "s/TRAEFIK_ADMIN_LOGIN=.*/TRAEFIK_ADMIN_LOGIN=$TRAEFIK_ADMIN_LOGIN/g" .env

TRAEFIK_ADMIN_PASS=$(date | md5sum | head -c 12)
while : ; do
  read  -r -e -p "Введите пароль администратора (не менее 12 символов): " -i "$TRAEFIK_ADMIN_PASS" TRAEFIK_ADMIN_PASS
  [ ${#TRAEFIK_ADMIN_PASS} -gt 11 ] && break
  echo "Введите пароль не менее 12 символов"
  TRAEFIK_ADMIN_PASS=""
done
sed -i -r "s|(TRAEFIK_ADMIN_PASS=).*|\1$TRAEFIK_ADMIN_PASS|" .env

TRAEFIK_PASSWORD=$(htpasswd -nb "$TRAEFIK_ADMIN_LOGIN" "$TRAEFIK_ADMIN_PASS" | sed -r 's/\$/\$\$/g')
echo "TRAEFIK_PASSWORD=$TRAEFIK_PASSWORD" >> .env

# Генерация email для получения сертификата от LE
if [ "${CERT:-1}" -eq 2 ];then
  while : ; do
    read -r -e -p "Введите email администратора: " -i "${ADMIN_EMAIL:-$TRAEFIK_ADMIN_LOGIN@$SERVICE_DOMAIN}" ADMIN_EMAIL
    [ "$ADMIN_EMAIL" ] && break
  done
  sed -i -r "s/ADMIN_EMAIL=.*/ADMIN_EMAIL=$ADMIN_EMAIL/g" .env
fi

echo "Данные обновлены в файле $workdir/.env"

if [ "$(docker compose ps -q | wc -l)" -gt 0 ]; then
  read -r -p "Требуется рестарт прокси. Выполнить? (Y/n): " ANSWER
  [[ "${ANSWER:-y}" =~ ^[yY]$ ]] || exit
  while :; do
    if docker compose up -d --force-recreate --wait; then
      echo  "Сервис прокси успешно перезапущен"
      break
    else
      read -r -p "Ошибка при рестарте. Повторить? (Y/n): " ANSWER
      [[ "${ANSWER:-y}" =~ ^[yY]$ ]] || exit 1
    fi
  done
fi