#!/bin/bash

getLEcert() {
  rm -rf data/
  mkdir data
  touch data/acme.json
  chmod 600 data/acme.json 
  cp conf/traefik-le.yml conf/traefik.yml
}

# DEBUG options
if [[ -n ${DEBUG:-} ]]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

set -euo pipefail

echo -e "\nНастраиваем прокси traefik\n"

if [[ -z $(which htpasswd) ]]; then
  echo "Устанавливаем apache2-utils для использования htpasswd"
  { apt-get update 2>&1 || echo E: update failed; } | grep -q '^[WE]:' && echo "Ошибка обновления, возможны ошибки при установке необходимых пакетов"
  apt-get -qq install apache2-utils 1> /dev/null || { echo "Ошибка установки"; exit 1; }
fi 

workdir=$(dirname $(realpath -s $0))
installdir=$(dirname "$workdir")
source "$installdir"/install/functions.sh &> /dev/null || { echo -e "FAIL\nОшибка подключения библиотеки функций из файла $installdir/install/functions.sh"; exit 1; }
cd "$workdir"

if [ ! -f .env ] || [[ ${1:-} = "--recreate" ]] || [[ ${1:-} = "-r" ]]; then
  
  [ -f docker-compose.yml ] || cp docker-compose-cert.yml docker-compose.yml
  [[ $(docker compose ps -aq | wc -l) -gt 0 ]] && docker compose down -v

  rm -f "$installdir"/.config && touch "$installdir"/.config
  cp template.env .env
  source .env
  [ -f "$installdir"/.env ] && source "$installdir"/.env

  # from install/functions.sh
  set_service_domain
  host_add
  host_add proxy

  chmod 600 .env
  IFS= read -erp "Введите логин администратора: " -i ${TRAEFIK_ADMIN_LOGIN:-promucadmin}
  TRAEFIK_ADMIN_LOGIN=${REPLY:-promucadmin}
  sed -i "s/TRAEFIK_ADMIN_LOGIN=.*/TRAEFIK_ADMIN_LOGIN=$TRAEFIK_ADMIN_LOGIN/g" .env
  sed -i "s/ADMIN_EMAIL=.*/ADMIN_EMAIL=$TRAEFIK_ADMIN_LOGIN@$SERVICE_DOMAIN/g" .env 
  # Генерация логина и пароля для авторизации в админке прокси
  TRAEFIK_ADMIN_PASS=$(date | md5sum | head -c 12)
  while :; do
    IFS= read  -erp "Введите пароль администратора (не менее 12 символов): " -i "$TRAEFIK_ADMIN_PASS" TRAEFIK_ADMIN_PASS
    [[ ${#TRAEFIK_ADMIN_PASS} -lt 12 ]] && { echo "Пароль менее 12 символов"; continue; } || break
  done
  sed -i -r "s|(TRAEFIK_ADMIN_PASS=).*|\1$TRAEFIK_ADMIN_PASS|" .env
  
  source .env
  echo -e "\nДля доступа к панели мониторинга прокси используйте\n\n\tURL: https://proxy.$SERVICE_DOMAIN\n\tлогин: $TRAEFIK_ADMIN_LOGIN\n\tпароль: $TRAEFIK_ADMIN_PASS\n" 

  TRAEFIK_PASSWORD=$(echo $(htpasswd -nb $TRAEFIK_ADMIN_LOGIN $TRAEFIK_ADMIN_PASS) | sed -r 's/\$/\$\$/g')
  echo "TRAEFIK_PASSWORD=$TRAEFIK_PASSWORD" >> .env

  # Генерация email для получения сертификата
  IFS= read -erp "Введите email администратора: " -i $ADMIN_EMAIL
  ADMIN_EMAIL=${REPLY:-$TRAEFIK_ADMIN_LOGIN@$SERVICE_DOMAIN}
  sed -i "s/ADMIN_EMAIL=.*/ADMIN_EMAIL=$ADMIN_EMAIL/g" .env

  echo "### Traefik ###" >> "$installdir"/.config
  echo "логин:  $TRAEFIK_ADMIN_LOGIN" >> "$installdir"/.config
  echo "пароль: $TRAEFIK_ADMIN_PASS" >> "$installdir"/.config
  echo -e "email:  $ADMIN_EMAIL" >> "$installdir"/.config

  rm -rf conf/dynamic/*.yml
  IFS= read -erp "Выберите - генерируем сертификат (1), получаем от Let's Encrypt (2) или использовать собственные (3): " CERT
  case ${CERT:-1} in
    1 ) cp docker-compose-cert.yml docker-compose.yml; ./cert.sh 1 ;;
    2 ) cp docker-compose-le.yml docker-compose.yml; getLEcert ;;
    3 ) cp docker-compose-cert.yml docker-compose.yml; ./cert.sh 3 ;;
  esac
  sed -i "s/CERT=.*/CERT=$CERT/g" .env
else
  echo -e "Возможно имеются данные, установка игнорируется.\nДля принудительной установки используйте флаг --recreate\n"
fi

# Старт прокси
echo -e "\nЗапускаем прокси"
[[ $(docker image ls -q traefik:"${TRAEFIK_TAG:-v2.9}" | wc -l) -eq 0 ]] && { "$installdir"/getimage.sh traefik:"${TRAEFIK_TAG:-v2.9}" || exit 1; }
if docker compose up -d --force-recreate --wait; then
  if [[ ! $CERT == "2" ]]; then
    docker cp -a traefik:/etc/ssl/certs/ca-certificates.crt conf/certs/ &>/dev/null || { echo "Ошибка при копировании ca-certificates.crt в контейнер прокси"; exit 2; }
    cat conf/certs/ca.crt >> conf/certs/ca-certificates.crt
  fi
else
  echo "Ошибка при запуске контейнера. Повторите установку."
  exit 1
fi