#!/bin/bash

# DEBUG options
if [ ! -z ${DEBUG:-} ]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

set -euo pipefail

echo -e "\nНастраиваем rabbitmq\n"

workdir=$(dirname $(realpath -s $0))
installdir=$(dirname $workdir)
source "$installdir"/install/functions.sh &> /dev/null || { echo -e "FAIL\nОшибка подключения библиотеки функций из файла $installdir/install/functions.sh"; exit 1; }
cd $workdir || exit 1


if [ ! -f .env ] || [[ ${1:-} = "--recreate" ]] || [[ ${1:-} = "-r" ]]; then
  [ ! -f docker-compose.yml ] && cp docker-compose-cert.yml docker-compose.yml
  [ $(docker compose ps -aq | wc -l) -gt 0 ] && docker compose down -v
  rm -rf data/ log/
  mkdir data log
  chown -R 999 data log

  cp template.env .env
  source .env
  [ -f ../.env ] && source ../.env

  # from install/functions.sh
  set_service_domain
  host_add mq
  
  echo -e "Доступ к RabbitMQ\n\n\tURL: https://mq.$SERVICE_DOMAIN\n\tлогин: $RABBITMQ_USERNAME\n\tпароль: $RABBITMQ_PASSWORD"

  echo -e "\n### RabbitMQ ###" >> ../.config
  echo "логин:  $RABBITMQ_USERNAME" >> ../.config
  echo "пароль: $RABBITMQ_PASSWORD" >> ../.config

else
  echo -e "Возможно имеются данные, установка игнорируется.\nДля принудительной установки удалите файл $(pwd)/.env или используйте флаг --recreate\n"
fi

echo -e "\nЗапускаем rabbitmq"
[ $(docker image ls -q rabbitmq:${RABBITMQ_TAG:-3.10-management} | wc -l) -eq 0 ] && { $installdir/getimage.sh rabbitmq:${RABBITMQ_TAG:-3.10-management} || exit 1; }
docker compose up -d --force-recreate --wait || { echo "Ошибка при запуске контейнера. Повторите установку."; exit 2; }