#!/bin/bash

# DEBUG options
if [[ -n ${DEBUG:-} ]]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

set -euo pipefail

echo -e "\nНастраиваем ruleengine\n"

workdir=$(dirname $(realpath -s $0))
installdir=$(dirname $workdir)
source "$installdir"/install/functions.sh &> /dev/null || { echo -e "FAIL\nОшибка подключения библиотеки функций из файла $installdir/install/functions.sh"; exit 1; }
cd "$workdir" || exit 1

if [ ! -f .env ] || [[ ${1:-} = "--recreate" ]] || [[ ${1:-} = "-r" ]]; then
  [ -f ../.env ] && source ../.env
  cp template.env .env
  
  # from install/functions.sh
  set_service_domain
  host_add re
  check_storage_volume
  
  [ ! -f docker-compose.yml ] && cp docker-compose-cert.yml docker-compose.yml
  if [ $(docker ps -q -f name=redis | wc -l) -eq 0 ]; then
    echo -e "Не запущен redis. Запустите и повторите установку ruleengine.\n"
    exit 1
  fi
  redis_ip=$(docker inspect -f '{{range.NetworkSettings.Networks}}{{.IPAddress}}{{end}}' redis)
  sed -i -r "s/(REDIS_HOST=).*/\1$redis_ip/" .env
  
  source .env  
  source $installdir/proxy/.env
  case ${CERT:-1} in
    2 ) cp docker-compose-le.yml docker-compose.yml ;;
    * ) cp docker-compose-cert.yml docker-compose.yml ;;
  esac

  # Выполняем первоначальную настройку репозитория
  source ./setup-ruleengine.sh

else
  echo -e "Возможно имеются данные, установка игнорируется.\nДля принудительной установки удалите файл $(pwd)/.env или используйте флаг --recreate\n"
fi

cd "$workdir" || exit 1

echo "Запускаем ruleengine..."

[ -f .env ] && source .env
[[ $(docker image ls -q psm/promuc_flow:"${RULEENGINE_TAG:?}" | wc -l) -eq 0 ]] && { "$installdir"/getimage.sh psm/promuc_flow:"${RULEENGINE_TAG:?}" || exit 1; }
[[ $(docker image ls -q psm/ruleengine:"${RULEENGINE_TAG:?}" | wc -l) -eq 0 ]] && { "$installdir"/getimage.sh psm/ruleengine:"${RULEENGINE_TAG:?}" || exit 1; }
[[ $(docker image ls -q psm/ruleengine-transport:"${RULEENGINE_TAG:?}" | wc -l) -eq 0 ]] && { "$installdir"/getimage.sh psm/ruleengine-transport:"${RULEENGINE_TAG:?}" || exit 1; }
[[ $(docker image ls -q psm/ruleengine-cov:"${RULEENGINE_TAG:?}" | wc -l) -eq 0 ]] && { "$installdir"/getimage.sh psm/ruleengine-cov:"${RULEENGINE_TAG:?}" || exit 1; }

if ! docker compose up -d --force-recreate --wait; then
  echo -e "FAIL\nОшибка при запуске контейнера. Повторите установку."
  exit 1
fi
