#!/bin/bash

source $installdir/proxy/.env
source $installdir/gitea/.env
source .env

if [ -z $(which jq) ] || [ -z $(which curl) ] || [ -z $(which git) ]; then
  echo "Устанавливаем jq curl git"
  { apt-get update 2>&1 || echo E: update failed; } | grep -q '^[WE]:' && echo "Ошибка обновления, возможны ошибки при установке необходимых пакетов"
  apt-get -qq install jq curl git 1> /dev/null || { echo "Ошибка установки"; exit 1; }
fi 

# подключаемся к gitea и проверяем её доступность
[ $(docker ps -q -f name=gitea | wc -l) -eq 0 ] && { echo -e "FAIL\nНе запущен сервис Gitea"; exit 1; }
[ ! -f $installdir/gitea/.env ] && { echo -e "FAIL\nОтсутствует файл настроек сервиса Gitea"; exit 1; }
source "$installdir"/gitea/check.sh
check_token_exist

# Проверям наличие орг promuc
responseCode=$(\
  curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X GET \
  https://git.$SERVICE_DOMAIN/api/v1/orgs?token=$token \
  -H 'accept: application/json'       \
  -H 'Content-Type: application/json'
)
[[ ! "$responseCode" == "200" ]] && { echo -e "Ошибка получения списка организаций\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"; failed; }
if [[ $(jq '.[] | select(.username=="promuc")' .apiresponse | wc -l) -eq 0 ]]; then
  # Создаём организацию
  echo -n "Создаём организацию promuc..."
  if ! responseCode=$(\
    curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X POST \
    https://git.$SERVICE_DOMAIN/api/v1/admin/users/$GITEA_ADMIN_USER/orgs?token=$token \
    -H 'accept: application/json'       \
    -H 'Content-Type: application/json' \
    -d '{ 
    "description": "Демо данные PromUC", 
    "full_name": "PromUC",
    "location": "", 
    "repo_admin_change_team_access": true, 
    "username": "promuc",      
    "visibility": "public",      
    "website": "https://'$SERVICE_DOMAIN'"}'
  ); then
    echo -e "FAIL\nОшибка при создании организации promuc\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"
    failed
  fi  
  if [[ "$responseCode" == "201" ]]; then
    echo "ОК"
  else
    echo -e "FAIL\nОшибка при создании организации promuc\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"
    failed
  fi  
fi

echo -n "Проверяем наличие репозитория psm/$RULEENGINE_REPO..."
if ! responseCode=$(\
  curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X GET \
  https://git.$SERVICE_DOMAIN/api/v1/repos/promuc/$RULEENGINE_REPO?token=$token \
  -H 'accept: application/json'       \
  -H 'Content-Type: application/json'
); then
  echo -e "FAIL\nОшибка при проверке наличия репозитория $RULEENGINE_REPO\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"
  failed
fi
if [[ "$responseCode" == "200" ]]; then
  echo -en "ОК\nУдаляем репозиторий psm/$RULEENGINE_REPO..."
  if ! responseCode=$(\
    curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X DELETE \
    https://git.$SERVICE_DOMAIN/api/v1/repos/promuc/$RULEENGINE_REPO?token=$token \
    -H 'accept: application/json'       \
    -H 'Content-Type: application/json'
  ); then
    echo -e "FAIL\nОшибка при удалении репозитория $RULEENGINE_REPO\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"
    failed
  fi
  if [[ "$responseCode" == "204" ]]; then
    echo "OK"
  else
    echo -e "FAIL\nОшибка при удалении репозитория $RULEENGINE_REPO\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"
    failed
  fi
else
  echo "OK"
fi

echo -n "Создаём репозиторий $RULEENGINE_REPO..."
if ! responseCode=$(\
  curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X POST \
  https://git.$SERVICE_DOMAIN/api/v1/admin/users/promuc/repos?token=$token \
  -H 'accept: application/json'       \
  -H 'Content-Type: application/json' \
  -d '{
    "auto_init": true,
    "default_branch": "main",
    "description": "Репозиторий для хранения конфигураций RuleEngine",
    "gitignores": "",
    "issue_labels": "",
    "license": "",
    "name": "'$RULEENGINE_REPO'",
    "private": true,
    "readme": "",
    "template": false,
    "trust_model": "default"
  }'
); then
  echo -e "FAIL\nОшибка при создании репозитория $RULEENGINE_REPO\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"
  failed
fi
if [[ "$responseCode" == "201" ]]; then
  echo "OK"
else
  echo -e "FAIL\nОшибка при создании репозитория $RULEENGINE_REPO\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"
  failed
fi

rm -rf data
mkdir -p data
[[ "$CERT" == "2" ]] && sslCAinfo="" || sslCAinfo="-c http.sslCAinfo=/etc/ssl/certs/ca-certificates.crt"
echo -n "Создаём папку configs в репозитории..."

git clone -q $sslCAinfo -c remote.origin.proxy="" -c user.name="$GITEA_ADMIN_USER" -c user.email="$ADMIN_EMAIL" -c remote.origin.url=https://$token@git."$SERVICE_DOMAIN"/promuc/"$RULEENGINE_REPO".git https://"$token"@git."$SERVICE_DOMAIN"/promuc/"$RULEENGINE_REPO".git data/"$RULEENGINE_REPO" || { echo -e "FAIL\nОшибка клонирования репозитория"; exit 1; }
cd data/"$RULEENGINE_REPO" || exit 1
mkdir configs
touch configs/.gitkeep
git add . 1> /dev/null
git commit -m "Создана папка configs" 1> /dev/null
git push origin main &> /dev/null && echo "ОК" || { echo -e "FAIL\nОшибка добавления в репозиторий"; exit 1; }

cd "$workdir" || exit 1
# Добавляем webhook для ruleengine-config
echo -n "Добавляем webhook для репозитория $RULEENGINE_REPO..."
if ! responseCode=$(\
  curl -s --noproxy "*" -o .apiresponse -w "%{http_code}\n" -X POST \
  https://git.$SERVICE_DOMAIN/api/v1/repos/promuc/$RULEENGINE_REPO/hooks?token=$token \
  -H 'accept: application/json'       \
  -H 'Content-Type: application/json' \
  -d '{
  "active": true,
  "config": {
    "url": "http://updater:9000/hooks/re-conf-update",
    "content_type": "json"
  },
  "events": [
    "push"
  ],
  "type": "gogs"
  }'); then
  echo -e "FAIL\nОшибка при настройке webhook в репозитории $RULEENGINE_REPO\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"
  failed
fi
if [[ "$responseCode" == "201" ]]; then
  echo "OK"
else
  echo -e "FAIL\nОшибка при настройке webhook в репозитории $RULEENGINE_REPO\nКод: $responseCode\nСообщение: $(cat .apiresponse | jq -j .message)"
  failed
fi