#!/bin/bash

# DEBUG options
if [ ! -z ${DEBUG:-} ]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

set -euo pipefail

echo -e "\nНастраиваем SCADA\n"

workdir=$(dirname $(realpath -s $0))
installdir=$(dirname $workdir)
source "$installdir"/install/functions.sh &> /dev/null || { echo -e "FAIL\nОшибка подключения библиотеки функций из файла $installdir/install/functions.sh"; exit 1; }
cd "$workdir" || exit 1

if [ -z $(which jq) ] || [ -z $(which curl) ]; then
  echo "Устанавливаем jq и curl"
  { apt-get update 2>&1 || echo E: update failed; } | grep -q '^[WE]:' && echo "Ошибка обновления, возможны ошибки при установке необходимых пакетов"
  apt-get -qq install jq curl 1> /dev/null || { echo "Ошибка установки"; exit 1; }
fi 

if [ ! -f .env ] || [[ ${1:-} = "--recreate" ]] || [[ ${1:-} = "-r" ]]; then
  
  cp template.env .env
  source .env
  [[ -f $installdir/.env ]] && source "$installdir"/.env

  [ ! -f docker-compose.yml ] && cp docker-compose-cert.yml docker-compose.yml
  [[ $(docker compose ps -aq | wc -l) -gt 0 ]] && docker compose down
  
  # from install/functions.sh
  set_service_domain
  host_add
  stop_storage_volume

  source .env
  source $installdir/proxy/.env
  case ${CERT:-1} in
    2 ) cp docker-compose-le.yml docker-compose.yml ;;
    * ) cp docker-compose-cert.yml docker-compose.yml ;;
  esac

  [ ! -d .secrets ] && mkdir .secrets
  [ -d .secrets/apps.json ] && rm -rf .secrets/apps.json
  touch .secrets/client_id .secrets/client_secret .secrets/apps.json .secrets/chpt
  [[ $(wc -l < .secrets/apps.json) -eq 0 ]] && echo '{}' > .secrets/apps.json
  chown 33 .secrets/apps.json .secrets/chpt
  [[ -z $APPS ]] && { echo "Не заполнено значение APPS в файле $workdir/.env"; exit 1; }
  
  echo "Настраиваем приложение в Gitea для аутентификации"
  [[ $(docker ps -q -f name=gitea | wc -l) -eq 0 ]] && { echo "Не запущен сервис Gitea"; exit 1; }
  # получаем токен
  source "$installdir"/gitea/check.sh
  check_token_exist
  
  # создаём приложения в Gitea
  readarray -td';' apps <<<"$APPS;"; unset 'apps[-1]'
  for appurl in "${apps[@]}"; do
    redirect_uris="$(echo "$appurl" | sed -nr "s/.+\|(.+)/\1/p")/" 
    servicename=$redirect_uris
    path=$(echo $redirect_uris | sed -nr 's/^[^\/]+\/(.+)$/\1/p' | sed 's/\//_/')
    [ ! -z $path ] && path='_'$path
    source $installdir/gitea/createapp.sh
    appsfile=$(cat ./.secrets/apps.json)
    echo "$appsfile" | jq '. + { 
      "https://'"$redirect_uris"'": {
        "client_id": "'"$client_id"'",
        "client_secret": "'"$client_secret"'",
        "access_token": "'"https://git.$SERVICE_DOMAIN/login/oauth/access_token"'",
        "redirect_url": "https://'"$redirect_uris"'",
        "api_url": "'"https://git.$SERVICE_DOMAIN/api/v1"'",
        "auth_url": "'"https://git.$SERVICE_DOMAIN/login/oauth/authorize?client_id={CLIENT_ID}&redirect_uri={REDIRECT_URL}&response_type=code&state={STATE}"'"
      }}' > ./.secrets/apps.json
    rm -f ./.secrets/client_*
  done
    
  echo -e "\n### SCADA ###" >> "$installdir"/.config
  echo "логин:  $(sed -n 's|GITEA_ADMIN_USER=||p' ../gitea/.env)" >> "$installdir"/.config
  echo "пароль: $(sed -n 's|TRAEFIK_ADMIN_PASS=||p' ../proxy/.env)" >> "$installdir"/.config

  # добавляем стандартный webhook
  "$workdir"/webhook.sh
  # импортируем репозитории
  "$workdir"/import-repos.sh -f "$workdir"/repos.yml

else
  echo -e "Возможно имеются данные, установка игнорируется.\nДля принудительной установки удалите файл $(pwd)/.env или используйте флаг --recreate\n"
  source .env
fi
cd "$workdir"
source $installdir/proxy/.env
if [[ ! "$CERT" == "2" ]]; then 
  [ -d ca.crt ] && rm -rf ca.crt
  if [[ -f $installdir/proxy/conf/certs/ca.crt ]]; then
    cp -p "$installdir"/proxy/conf/certs/ca.crt .
  else
    echo -e "Отсутствует файл ca.crt в каталоге прокси\nВозможно прокси не был установлен или произошла ошибка при его запуске, повторите его настройку\nНастройка прервана\n"
    exit 1
  fi
  [ -d ca-certificates.crt ] && rm -rf ca-certificates.crt
  if [[ -f $installdir/proxy/conf/certs/ca-certificates.crt ]]; then
    cp -p "$installdir"/proxy/conf/certs/ca-certificates.crt .
  else
    echo -e "Отсутствует файл сертификатов ca-certificates.crt в каталоге прокси\nВозможно прокси не был установлен или произошла ошибка при его запуске, повторите его настройку\nНастройка прервана\n"
    exit 1
  fi
fi

echo -e "\nЗапускаем scada\n"
[ $(docker image ls -q psm/promuc_scadag:latest | wc -l) -eq 0 ] && { "$installdir"/getimage.sh psm/promuc_scadag:latest || exit 1; }
docker compose up -d --force-recreate --wait || { echo "Ошибка при запуске контейнера. Повторите установку."; exit 2; }