#!/bin/bash
PS4='$LINENO: '
#set -x
[ "$#" -lt 2 ] && exit 1

function pull {
  cd "$path" || exit 1
  git config remote.origin.url http://"$token"@gitea:3000/"$owner"/"$repo".git
  git config pull.rebase true
  git switch -q "$branch"
  git reset -q --hard
  git pull --rebase -q origin && echo "success" || echo "failed"
  git config remote.origin.url http://gitea:3000/"$owner"/"$repo".git
}

function clone {
  mkdir -p "$path"
  git clone -q --single-branch --branch "$branch" -c pull.ff=only http://"$token"@gitea:3000/"$owner"/"$repo".git "$path" && echo "success" || echo "failed"
}

function push {
  cd "$path" || exit 1
  git config remote.origin.url http://"$token"@gitea:3000/"$owner"/"$repo".git
  git config user.name "admin"
  git config user.email "admin@${SERVICE_DOMAIN:-localhost}"
  git add .
  git commit -m "Фиксация изменений с сайта"
  git push origin && echo "success" || echo "failed"
  git config remote.origin.url http://gitea:3000/"$owner"/"$repo".git
}

function delete {
  rm -rf ./"${owner:?}"/"$repo"/ && echo "success" || echo "failed"
  if [[ $(find ./"$owner"/ -mindepth 1 -maxdepth 1 -type d | wc -l) -eq 0 ]]; then
    rm -rf "${owner:?}"/ && echo "Каталог $owner/ удалён из кеша" || echo "Can't delete $owner/"
  fi
}

workdir=$(pwd)
repo="$1"
owner="$2"
branch="${3:-main}"
path="${owner:?}"/"$repo"/"$branch"

[ -f /run/secrets/gitea_user_token ] || exit 1
token=$(cat /run/secrets/gitea_user_token)
default_branch=$(curl -ksf --noproxy "*" https://git.$SERVICE_DOMAIN/api/v1/repos/$owner/$repo?token=$token | jq -r '.default_branch') || { echo "Ошибка получения default_branch"; exit 2; }

if [ -n "$REF" ]; then
  if [[ "$REF_TYPE" = "branch" ]]; then
    branch="$REF"
  else
    branch=$(echo "$REF" | sed -nr 's/refs\/heads\/(.+)/\1/p')
  fi
  case "$X_GITEA_EVENT_TYPE" in
    push|create) 
      if [ -d "$path"/.git ]; then
        if [[ ${X_SENDER:-} = ruleengine ]]; then
          push
        else
          pull
        fi
      else
        clone
      fi ;;
    delete)
      if rm -rf "$path"; then
        echo "success"
        exit 0
      else
        echo "failed"
        exit 1
      fi
  esac
elif [ -n "$ACTION" ]; then
  case "$ACTION" in
    created) clone  ;;
    deleted) delete ;;
  esac
fi

# контейнеры
cd $workdir
if ! ${TEMPLATE:-false} && \
   [ -f "$path"/plugin.yml ] && \
   grep -qx "type: container" "$path"/plugin.yml && \
   [[ $branch = "$default_branch" ]] \
  ; then
  curl -s --noproxy "*" -H "Content-Type:application/json" -H "X-Gitea-Event-Type:$X_GITEA_EVENT_TYPE" -d '{"repo":"'"$repo"'","owner":"'"$owner"'","branch":"'"$branch"'","action":"'"$ACTION"'"}' http://promuc-updater:9090/hooks/container
fi