#!/bin/bash 

#PS4='$LINENO: '
#set -x

function container_stop {
  jq '.plugins -= ["'"$image_name"'"]' "$installdir"/install/config.json > config.json.tmp && mv config.json.tmp "$installdir"/install/config.json
  if [ "$(docker ps -a -q -f name="$image_name" | wc -l)" -gt 0 ]; then
    if docker stop "$image_name" 1> /dev/null; then
      if docker rm -fv "$image_name" 1> /dev/null; then
        echo -n "success stop and delete $image_name"
      else
        echo -n "success stop but fail delete $image_name"
        exit 1
      fi
    else
      echo -n "fail stop $image_name"
      exit 1
    fi
  fi
}

function container_start {
  if [ "$(docker ps -a -q -f name="$image_name" | wc -l)" -gt 0 ]; then
    container_stop
  fi
  cd "$path" || exit 1
  jq '.plugins += ["'"$image_name"'"]' "$installdir"/install/config.json > config.json.tmp && mv config.json.tmp "$installdir"/install/config.json 
  if docker build -q \
                 --build-arg APP_FORKS="$plugin_forks" \
                 --build-arg PLUGIN_PORT="$plugin_port" \
                 -t "$image_name" . 1> /dev/null; then
    if docker run \
      --name "$image_name" \
      --hostname "$image_name" \
      --detach \
      --network proxy_backend \
      --expose "$plugin_port" \
      --restart=on-failure:2 \
      --user 1 \
      "$image_name" 1> /dev/null; then
      echo -n "success build and start $image_name"
      exit 0
    else
      echo -n "success build but failed start $image_name"
      exit 1  
    fi
  else
    echo -n "failed build $image_name"
    exit 1
  fi
}

workdir=$(dirname "$(realpath -s $0)")
installdir=$(dirname "$(dirname $workdir)")
source "$installdir"/install/functions.sh &> /dev/null || { echo -e "FAIL\nОшибка подключения библиотеки функций из файла $installdir/install/functions.sh"; exit 1; }
promuc_storage_path=$(docker inspect -f '{{.Mountpoint}}' promuc_storage)
[[ -f $installdir/install/config.json ]] || create_config

repo="$1"
owner="$2"
branch="$3"
path="$promuc_storage_path"/"${owner:?}"/"$repo"/"$branch"
image_name=$(echo plugin_"$owner"_"$repo" | tr '[:upper:]' '[:lower:]')

[[ "$ACTION" = delete ]] && container_stop

source "$installdir"/install/functions.sh &> /dev/null || { echo -e "FAIL\nОшибка подключения библиотеки функций из файла $installdir/install/functions.sh"; exit 1; }
eval "$(parse_yaml "$path"/plugin.yml 'plugin_')"

case $plugin_run in
  true ) container_start ;;
  false ) container_stop ;;
  * )
      echo "Не верное значение run в plugin.yml"
      exit 1
  ;;
esac