#!/bin/bash

# DEBUG options
if [[ -n ${DEBUG:-} ]]; then
  exec 5> $(dirname $(realpath -s $0))/.$(basename $0).log
  BASH_XTRACEFD="5"
  PS4='$LINENO: '
  set -x
fi

workdir=$(dirname "$(realpath -s $0)")
installdir=$(dirname "$(dirname $workdir)")
cd "$workdir" || exit 1

if [ -n ${1:-} ]; then
  plugins=$1
else
  config="$installdir"/install/config.json
  [[ -f $config ]] || exit 0
  mapfile -t plugins < <(jq -r '.plugins[]' "$config")
fi
for plugin in "${plugins[@]}"; do
  if [ "$(docker ps -q -f name="$plugin" | wc -l)" -gt 0 ]; then
    continue
  else
    docker rm "$plugin" &>/dev/null
  fi
  if docker run \
      --name "$plugin" \
      --hostname "$plugin" \
      --detach \
      --network proxy_backend \
      --restart=on-failure:2 \
      --user 1 \
      "$plugin" &>/dev/null; then
    echo -n "success start $plugin"
  else
    echo -n "failed start $plugin"
  fi
done